package Comment2CSV::CMS;
use strict;

use MT::Comment;
use MT::Util qw( offset_time_list );

sub _download_entry_comment_csv {
    return _download_comment_csv( @_ );
}

sub _download_comment_csv {
    my $app = shift;
    $app->validate_magic or
        return $app->trans_error( 'Permission denied.' );
    my $blog = $app->blog;
    return $app->trans_error( 'Invalid request' ) if (! $blog );
    return $app->trans_error( 'Invalid request' ) if (! $blog->is_blog );
    my $plugin = MT->component( 'Comment2CSV' );
    my $user = $app->user;
    my $can_access = $user->is_superuser;
    $can_access = $user->permissions( $blog->id )->can_administer_blog if (! $can_access );
    $can_access = $user->permissions( $blog->id )->can_manage_feedback if (! $can_access );
    if (! $can_access ) {
        return $app->trans_error( 'Permission denied.' );
    }
    my $column_names = MT::Comment->column_names;
    eval { require CustomFields::Field; };
    if (! $@ ){
        my @fields = CustomFields::Field->load( { blog_id => $blog->id, obj_type => 'comment' } );
        for my $field ( @fields ) {
            push ( @$column_names, 'field.' . $field->basename );
        }
    }
    my $convert2 = $plugin->get_config_value( 'convert2' );
    $convert2 = lc( $convert2 ) if $convert2;
    my $csv = do {
        eval { require Text::CSV_XS };
        unless ( $@ ){
            Text::CSV_XS->new ( { binary => 1 } );
        }else{
            eval { require Text::CSV };
            return $app->trans_error( 'Neither Text::CSV_XS nor Text::CSV is available.' ) if $@;
            Text::CSV->new ( { binary => 1 } );
        }
    };
    my @comments;
    if ( $app->param( 'action_name' ) eq 'download_comment_csv' ) {
        my @ids = $app->param( 'id' );
        for my $id ( @ids ) {
            next unless $id;
            my $comment = MT::Comment->load( { id => $id, blog_id => $blog->id } );
            return $app->trans_error( 'Invalid request' ) if (! $comment );
            push ( @comments, $comment );
        }
    } else {
        my @entry_ids = $app->param( 'id' );
        require MT::Entry;
        my @entries = MT::Entry->load( { id => \@entry_ids, blog_id => $blog->id } );
        for my $entry ( @entries ) {
            my $comment = $entry->comments;
            next unless ( defined  $comment );
            push ( @comments, @$comment );
        }
    }
    $app->{ no_print_body } = 1;
    my @tl = offset_time_list( time, $blog );
    my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
    $app->set_header( "Content-Disposition" => "attachment; filename=comment2csv_$ts.csv" );
    $app->set_header( "pragma" => '' );
    $app->send_http_header( 'text/csv' );
    if ( $csv->combine( @$column_names ) ) {
        my $string = $csv->string;
        $app->print( $string );
    }
    for my $comment ( @comments ) {
        my @fields;
        for my $column ( @$column_names ) {
            my $value = $comment->$column;
            if ( ( $column =~ /_on$/ ) && ( $value =~ /^[0-9]{14}$/ ) ) {
                $value = "\t$value";
            }
            push ( @fields, $value );
        }
        if ( $csv->combine( @fields ) ) {
            my $string = $csv->string;
            if ( $convert2 ) {
                if ( $convert2 eq 'shift_jis' ) {
                    $string = utf8_string_to_cp932_octets( $string );
                } else {
                    $string = MT::I18N::encode_text( $string, 'utf8', $convert2 );
                }
            } else {
                $string = MT::I18N::utf8_off( $string );
            }
            $app->print( "\n$string" );
        }
    }
}

sub utf8_string_to_cp932_octets {
    my $str = shift;
    $str = MT::I18N::utf8_off( $str );
    Encode::from_to( $str, 'utf8', 'cp932' );
    return $str;
}

1;